require("Globals")


require("util")
local Gui = require("Gui")
local GuiEvents = require("GuiEvents")
local Tas = require("Tas");
local Sequence = require("Sequence")

-- taken from latest commit https://github.com/pkulchenko/serpent/blob/522a6239f25997b101c585c0daf6a15b7e37fad9/src/serpent.lua
local serpent_github_0_30 = require("serpent_github_0_30")
local serpent_github_0_301 = require("serpent_github_0_301")
local serpent_github_0_302 = require("serpent_github_0_302")

inspect = require("inspect")

script.on_init( function()
    -- Dont capture and print error, players won't see it as they haven't been added to the game
    -- Instead collect the traceback in err and rethrow to display it in the main menu.
    local _, err = xpcall(function()
        log_error({"TAS-info-generic", "Calling script.on_init"}, true)
        util.init_globals()
        global.gui_events = GuiEvents.new()
        global.gui = Gui.new(global.gui_events, Tas.new())

        -- ##########################################################################################################
        -- lets make a big nested structure as another test and also include a
        -- key `sequence`, to see if the options maxlevel or keyignore are being used
        global.foo2 = {
            foo3 = {
                foo4 = {
                    foo5 = {
                        foo6 = {
                            foo7 = {
                                foo8 = {
                                    foo9 = {
                                        foo10 = {
                                            foo11 = {
                                                foo12 = {
                                                    foo13 = {
                                                        foo14 = {
                                                            foo15 = {
                                                                foo16 = 6
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            },
            sequence = "my_sequence",
            actual_sequence_object = Sequence.new()
        }
        -- ##########################################################################################################


        Gui.set_metatable(global.gui)
        -- this is totally optional and increases new game and multiplayer join
        -- time, but catches some loading bugs

    end, debug.traceback, event)

    if err then
        error(err)
    end
end )

script.on_load( function()
    -- Dont capture and print error, printing will result in another error because `game` is nil.
    -- (resulting in loss of the initial stacktrace).
    -- Instead collect the traceback in err and rethrow to display it in the main menu.
    local _, err = xpcall(function() 
        log_error({"TAS-info-generic", "Calling script.on_load"}, true)

        -- ##########################################################################################################
        -- Check if global was deserialized properly. It doesn't but really should.
        -- by the way this sequence is also accesible via
        -- global.gui.tas.sequence_indexer.sequences[1]
        if global.gui.tas.playback_controller.runner_instances[1].runner.sequence ~= nil then
            log_error("global deserialized, using runner.sequence waypoint x coordinate is " .. global.gui.tas.playback_controller.runner_instances[1].runner.sequence.waypoints[1].position.x)
        else
            log_error("runner.sequence is nil")
        end

        -- access the sequence via sequence_indexer
        local dumped = serpent.dump(global--[[, {compact = false, indent = " ", comment = true}]])
        local ok, parsed = serpent.load(dumped)
        if global.gui.tas.sequence_indexer.sequences[1] ~= nil then
            log_error("global deserialized, using sequence_indexer.sequences[1] the waypoint x coordinate is " .. global.gui.tas.sequence_indexer.sequences[1].waypoints[1].position.x)
        else
            log_error("sequence_indexer.sequences[1] is nil")
        end

        -- check if our deeply nested foo structure got deserialized. It does,
        -- so this isn't caused from the `maxlevel` or `keyignore` options
        -- being set in serpent during game save
        log_error("foo structure deserialized, foo16 value is " .. global.foo2.foo3.foo4.foo5.foo6.foo7.foo8.foo9.foo10.foo11.foo12.foo13.foo14.foo15.foo16)
        log_error("foo.sequence deserialized, sequence value is " .. global.foo2.sequence)



        -- the call to Gui.set_metatable below will throw because the full global table isn't deserialized.
        -- The error output is below. If it loads or an unrelated error is thrown during load then the problem should be fixed
        --[[
            Error while running on_load:
            __FactorioTas__/control.lua:98:
            __FactorioTas__/Sequence.lua:12:
            bad argument #1 to 'setmetatable' (table expected, got nil)
        ]]
        -- ##########################################################################################################
        Gui.set_metatable(global.gui)
    end, debug.traceback, event)
    
    if err then
        error(err)
    end
end )

script.on_event(defines.events.on_player_created, function(event)
    local _, err = xpcall(function(event)
        global.gui:init_player(event.player_index)
    end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_player_created", err } end
end )

script.on_event(defines.events.on_gui_click, function(event)


    -- ##########################################################################################################
    -- set up buttons for this test
    if event.element == global.gui.players[event.player_index].editor_visible_toggle and btns_created ~= true then
        -- just use hacky global variables
        serialize_builtin = game.players[event.player_index].gui.top.add { type = "button", name = "serialize_builtin", caption = "Test Builtin v0.30" }
        serialize_github_0_30 = game.players[event.player_index].gui.top.add { type = "button", caption = "Test Github.com/Serpent v0.30" }
        serialize_github_0_301 = game.players[event.player_index].gui.top.add { type = "button", caption = "Test Github.com/Serpent v0.301" }
        serialize_github_0_302 = game.players[event.player_index].gui.top.add { type = "button", caption = "Test Github.com/Serpent v0.302" }
        compare_serpent_versions = game.players[event.player_index].gui.top.add { type = "button", name = "compare_serpent", caption = "Compare Serpent v0.30 Runtimes" }
        btns_created = true
    end
    
    if event.element == serialize_builtin then
        -- dump/load global to prove it should normally be savable 
        test_serpent(serpent.dump, serpent.load, global)
    end
    
    if event.element == serialize_github_0_30 then
        -- dump/load global to prove it should normally be savable 
        test_serpent(serpent_github_0_30.dump, serpent_github_0_30.load, global)
    end
    
    if event.element == serialize_github_0_301 then
        -- dump/load global to prove it should normally be savable 
        test_serpent(serpent_github_0_301.dump, serpent_github_0_301.load, global)
    end
    
    if event.element == serialize_github_0_302 then
        -- dump/load global to prove it should normally be savable 
        test_serpent(serpent_github_0_302.dump, serpent_github_0_302.load, global)
    end

    if event.element == compare_serpent_versions then
        local builtin = serpent_github_0_302.dump(serpent)
        local github = serpent_github_0_302.dump(serpent_github_0_30)

        game.write_file("serpent_builtin.txt", builtin)
        game.write_file("serpent_github_0.30.txt", github)

        log_error("Serpent written to script-output. Serpent versions match: " .. tostring(builtin == github))
    end
    -- ##########################################################################################################

    
    local _, err = xpcall(function (event)
        global.gui_events:on_click(event)
        -- gui:on_click is deprecated, use gui_events in the future
        global.gui:on_click(event)
    end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_gui_click", err } end
end )

function test_serpent(serialize_func, deserialize_func, global_object)
    local dumped = serialize_func(global_object--[[, {compact = false, indent = " ", comment = true}]])
    local ok, parsed = deserialize_func(dumped)
    if parsed.gui.tas.playback_controller.runner_instances[1].runner.sequence ~= nil then
        log_error("global deserialized, using runner.sequence the waypoint x coordinate is " .. parsed.gui.tas.playback_controller.runner_instances[1].runner.sequence.waypoints[1].position.x)
    else
        log_error("runner.sequence is nil")
    end
end

script.on_event(defines.events.on_gui_checked_state_changed, function(event)
    local _, err = xpcall(function (event) global.gui_events:on_check_changed(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_check_changed", err } end
end )

script.on_event(defines.events.on_gui_selection_state_changed, function(event)
    local _, err = xpcall(function (event) global.gui_events:on_dropdown_selection_changed(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_dropdown_selection_changed", err } end
end )

script.on_event(defines.events.on_built_entity, function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_built_entity(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_built_entity", err } end
end )

script.on_event(defines.events.on_pre_player_mined_item, function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_pre_mined_entity(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_preplayer_mined_item", err } end
end )

script.on_event(defines.events.on_robot_pre_mined, function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_pre_mined_entity(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_robot_pre_mined", err } end
end )

script.on_event(defines.events.on_player_crafted_item, function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_crafted_item(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_crafted_item", err } end
end )

script.on_event("tas-select-hotkey", function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_left_click(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "tas-select-hotkey", err } end
end )

script.on_event(defines.events.on_tick, function(event)
    local _, err = xpcall(function (event) global.gui.tas:on_tick(event) end, debug.traceback, event)
    if err then log_error { "TAS-exception-specific", "on_tick", err } end
end )